# Nihongo Benkyo
# Copyright (C) 2005 Mathieu Blondel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

require 'singleton'

module NihongoBenkyo

    DB_DIR = File.join(ENV['HOME'],".nihongobenkyo")
    DB_PATH  = File.join(DB_DIR,"dic.db")

    class Preferences
        include Singleton
    
        def initialize

            unless(FileTest.exists?(DB_DIR))
                Dir.mkdir(DB_DIR)    
            end
            
            unless(FileTest.exists?(DB_PATH))
                File.new(DB_PATH, File::CREAT).close
            end
        end
        
        def method_missing(id, *args)
            method = id.id2name
            if match = /(.*)=$/.match(method)
                if args.length != 1
                    raise "Set method #{method} should be called with " +
                          "only one argument (was called with #{args.length})"
                end
                Database.instance.set_config(match[1],args.first)
            else
                unless args.empty?
                    raise "Get method #{method} should be called " +
                          "without argument (was called with #{args.length})"
                end
                Database.instance.get_config(method)
            end                
        end
            
    end

end