# Nihongo Benkyo
# Copyright (C) 2005 Mathieu Blondel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

module NihongoBenkyo
module UI

    class AddDictionaryDialog < Gtk::FileChooserDialog
        include GetText
        GetText.bindtextdomain(NihongoBenkyo::TEXTDOMAIN, nil, nil, "UTF-8")
        
        def initialize(parent)
            super()
            self.title = _("Add a dictionary")
            self.action = Gtk::FileChooser::ACTION_OPEN
            self.transient_for = parent.preferences_dialog
            self.modal = true
            @parent = parent
            @db = Database.instance
            supported = ["Edict", "JMDict", "JMnedict"]
            langs = ["de", "en", "fr", "ru"]
            self.current_folder = ENV['HOME']
                       
            add_button(Gtk::Stock::CANCEL, Gtk::Dialog::RESPONSE_CANCEL)
            add_button = add_button(Gtk::Stock::APPLY, 
                                       Gtk::Dialog::RESPONSE_ACCEPT)
            add_button.sensitive = false
                 
            self.signal_connect('destroy') { hide }
            
            vbox = Gtk::VBox.new
            vbox.spacing = 15
            
            # Addding dic name to the file chooser
            
            hbox = Gtk::HBox.new
            name_entry = Gtk::Entry.new
            
            name_entry.signal_connect('changed') do
                add_button.sensitive = !name_entry.text.strip.empty?
            end
            
            name_entry.signal_connect("activate") do
                add_button.clicked
            end
            
            name_label = Gtk::Label.new(_("Dictionary name:"), true)
            name_label.xalign = 0
            name_label.mnemonic_widget = name_entry
            hbox.pack_start(name_label)
            hbox.pack_start(name_entry)
            hbox.show_all
            vbox.pack_start(hbox)
            
            # Adding dic type to the file chooser
            
            hbox = Gtk::HBox.new
            type_combo = Gtk::ComboBox.new
                        
            supported.each { |s| type_combo.append_text(s) }
            type_combo.active = 0
            name_label = Gtk::Label.new(_("Dictionary type:"), true)
            name_label.xalign = 0
            name_label.mnemonic_widget = type_combo
            hbox.pack_start(name_label)
            hbox.pack_start(type_combo)
            hbox.show_all
            vbox.pack_start(hbox)
            
            # Adding langs to the file chooser
            
            checkbox = {}
            table = Gtk::Table.new(1, langs.length, true)
            
            n = 0
            langs.each do |lang|
                checkbox[lang] = Gtk::CheckButton.new(lang)
                checkbox[lang].active = true
                table.attach_defaults(checkbox[lang], n, n+1, 0, 1)
                n += 1
            end
            
            table.show_all
            table.visible = false
            
            type_combo.signal_connect("changed") do
                if supported[type_combo.active] == "JMDict"
                    @langs_hbox.visible = true
                else
                    @langs_hbox.visible = false
                end
            end
            
            @langs_hbox = Gtk::HBox.new
            name_label = Gtk::Label.new(_("Languages to import:"), true)
            name_label.xalign = 0
            name_label.mnemonic_widget = table
            @langs_hbox.pack_start(name_label)
            @langs_hbox.pack_start(table)
            @langs_hbox.show_all
            @langs_hbox.visible = false
            vbox.pack_start(@langs_hbox)
                     
            self.extra_widget = vbox
                       
            while run == Gtk::Dialog::RESPONSE_ACCEPT
                if name_entry.text.empty?
                    ErrorDialog.new(self, _("A dictionary name must be specified"))
                elsif (not name_entry.text.index(" ").nil?)
                    ErrorDialog.new(self, _("A dictionary name cannot use spaces"))
                elsif @db.dictionary_exists?(name_entry.text)
                    ErrorDialog.new(self, _("Dictionary name already exists"))
                else
                   
                    # The user must not interact with the add_dictionary_dialog
                    # during the import process 
                    
                    self.sensitive = false
                    import_completed = false
                    @db.close
                    
                    # It is much quicker to first drop the indexes,
                    # then import the data and finally recreate the indexes 
                    
                    # drop the indexes
                    
                    thread = Thread.new do 
                        `nihongobenkyo-import --drop-indexes #{NihongoBenkyo::DB_PATH}`    
                    end
                    
                    pbar = ProgressBar.new(_("Dropping database indexes..."), self)
                    pbar.sensitive = false 
           
                    while thread.alive?
                        pbar.pulse   
                    end
                    
                    pbar.destroy unless pbar.destroyed?
                    
                    # import data
                    
                        # generate the command line
                                         
                    cmd = "nihongobenkyo-import" 
                    cmd += " --#{supported[type_combo.active].downcase} #{self.filename}"
                    cmd += " --dic-name #{name_entry.text}"
                    
                    checkbox.each_pair do |lang, checkbox|
                        if checkbox.active?
                            cmd += " --lang #{lang}"
                        end
                    end if supported[type_combo.active] == "JMDict"
                    
                    cmd += " #{NihongoBenkyo::DB_PATH}"
                    
                        # execute the command

                    thread = Thread.new do
                        `#{cmd}`
                    end
                    
                    pbar = ProgressBar.new(_("Adding dictionary..."), self) 
                    pbar.signal_connect("destroy") do
                        `killall nihongobenkyo-import`
                        thread.kill 
                    end
           
                    while thread.alive?
                        pbar.pulse   
                    end
                    
                    unless pbar.destroyed?
                        pbar.destroy
                        import_completed = true 
                    end
                    
                    # recreate the indexes
                    
                    thread = Thread.new do 
                        `nihongobenkyo-import --create-indexes #{NihongoBenkyo::DB_PATH}`    
                    end
                    
                    pbar = ProgressBar.new(_("Creating database indexes..."), self) 
                    pbar.sensitive = false
           
                    while thread.alive?
                        pbar.pulse   
                    end
                    
                    pbar.destroy unless pbar.destroyed?
                    
                    # Done !
                    self.sensitive = true
                    @db.open
                    
                    if import_completed   
                        if @db.dictionary_exists?(name_entry.text)
                            @parent.append_dictionary(name_entry.text)
                        else 
                            ErrorDialog.new(self, _("Could not import dictionary"))
                        end
                    end
                    
                    break                                 
                end
            end
            
            self.destroy
        end    
    
    end

end
end